#+TITLE: Prise en main de Guix — UST4HPC 2021
#+AUTHOR: Ludovic Courtès
#+DATE: 19 janvier 2021

* Préliminaires

  - [[https://guix.gnu.org/guix-refcard.pdf][feuille de référence]]
  - [[https://guix.gnu.org/manual/fr/html_node][manuel de référence]]

* Installation

  - [[https://guix.gnu.org/manual/fr/html_node/Installation-binaire.html][installation binaire]]
  - Debian & co.: =apt install guix=
  - [[https://hpc.guix.info/blog/2017/11/installing-guix-on-a-cluster/][installation sur cluster]]
  - [[https://guix.gnu.org/en/download/][page téléchargement]]

* Recherche puis installation GCC + OpenMPI

  #+begin_src sh
    guix install gcc-toolchain openmpi
  #+end_src

* Compilation de code MPI

  #+begin_src c
    #include <mpi.h>
    #include <stdio.h>

    int main(int argc, char** argv) {
      // Initialize the MPI environment
      MPI_Init(NULL, NULL);

      // Get the number of processes
      int world_size;
      MPI_Comm_size(MPI_COMM_WORLD, &world_size);

      // Get the rank of the process
      int world_rank;
      MPI_Comm_rank(MPI_COMM_WORLD, &world_rank);

      // Get the name of the processor
      char processor_name[MPI_MAX_PROCESSOR_NAME];
      int name_len;
      MPI_Get_processor_name(processor_name, &name_len);

      // Print off a hello world message
      printf("Hello world from processor %s, rank %d out of %d processors\n",
	     processor_name, world_rank, world_size);

      // Finalize the MPI environment.
      MPI_Finalize();
    }

  #+end_src

* Lancement de bancs d'essai Intel (IMB) à la volée

  #+begin_src sh
    guix environment --ad-hoc openmpi intel-mpi-benchmarks -- mpirun -np 2 IMB-P2P PingPong
  #+end_src

* De même avec MPICH (option de transformation de paquets)

  #+begin_src sh
    guix environment --ad-hoc openmpi intel-mpi-benchmarks	\
	 --with-input=openmpi=mpich				\
	 -- mpirun -np 2 IMB-P2P PingPong
  #+end_src

  #+begin_src sh
    guix build --help-transform
  #+end_src

* Environnement interactif (hors/dans conteneur)

  #+begin_src sh
    guix environment --ad-hoc gcc-toolchain openmpi -C
  #+end_src

* Déclaration, « manifeste »

  #+begin_src sh
    cat > "manifest.scm" <<EOF
    (specifications->manifest
     (list "gcc-toolchain" "openmpi" "intel-mpi-benchmarks"))
    EOF

    guix package -m manifest.scm
  #+end_src

* Mise à jour et canaux

  #+begin_src sh
    guix pull
  #+end_src

  #+begin_src sh
    mkdir -p ~/.config/guix
    cat > ~/.config/guix/channels <<EOF
    ;; Ajout du canal Guix-HPC (Inria) et de Guix-Science.

    (append (list (channel
		   (name 'guix-science)
		   (url "https://github.com/guix-science/guix-science.git")
		   (introduction
		    (make-channel-introduction
		     "b1fe5aaff3ab48e798a4cce02f0212bc91f423dc"
		     (openpgp-fingerprint
		      "CA4F 8CF4 37D7 478F DA05  5FD4 4213 7701 1A37 8446"))))
		  (channel
		   (name 'guix-hpc)
		   (url "https://gitlab.inria.fr/guix-hpc/guix-hpc.git")))
	    %default-channels)
    EOF

    guix pull
  #+end_src

* Définition de paquets

  #+begin_src scheme
    guix import pypi itsdangerous
  #+end_src

  https://guix.gnu.org/manual/fr/html_node/Definition-des-paquets.html
